-- $Id: GraphDisplayInstanceTests.hs,v 1.4 2013/03/31 21:14:30 leavens Exp $
module GraphDisplayInstanceTests where
import GraphDisplayInstance
import GraphDisplay
import Control.Monad (forM_)
import Testing

main :: IO ()
main = do heading
          run_tests tests

heading :: IO ()
heading = do startTesting "GraphDisplayInstancesTests $Revision: 1.4 $"

debugIt :: IO ()
debugIt = do forM_ graphs (\(title, gr) -> do putStr title
                                              print (graph2display gr))

graphs :: [(String,Graph)]
graphs = [("y=3", Section {size = 4, center = (0,3), fun = (\_ -> 3)})
         ,("y=x", Section {size = 4, center = (0,0), fun = id})
         ,("y=x+3", Section {size = 10, center = (-3,0), fun = (\x -> x+3)})
         ,("y=square x", Section {size = 24, center = (0,9), fun = square})
         ,("y=cube x", Section {size = 30, center = (6,6), fun = cube})
         ,("y=sine x", Section {size = 20, center = (0,0), fun = sine})
         ]

mbv :: Maybe b -> b
mbv mb = case mb of 
           Just x -> x
           Nothing -> undefined

run2string :: String -> String
run2string title = show (graph2display (mbv (lookup title graphs)))

tests :: [TestCase String]
tests = [eqTest (run2string "y=3")
           "==" ("\n  5     \n"
                 ++"  4     \n"
                 ++"y 3*****\n"
                 ++"  2     \n"
                 ++"  1     \n"
                 ++"  -2 0 2\n"
                 ++"     x\n")
        ,eqTest (run2string "y=x")
         "==" ("\n   2    *\n"
               ++"   1   * \n"
               ++"y  0  *  \n"
               ++"  -1 *   \n"
               ++"  -2*    \n"
               ++"   -2 0 2\n"
               ++"      x\n")
        ,eqTest (run2string "y=x+3")
         "==" ("\n   5          *\n"
               ++"   4         * \n"
               ++"   3        *  \n"
               ++"   2       *   \n"
               ++"   1      *    \n"
               ++"y  0     *     \n"
               ++"  -1    *      \n"
               ++"  -2   *       \n"
               ++"  -3  *        \n"
               ++"  -4 *         \n"
               ++"  -5*          \n"
               ++"   -8-6-4-2 0 2\n"
               ++"         x\n")
        ,eqTest (run2string "y=square x")
         "==" ("\n  21                         \n"
               ++"  20                         \n"
               ++"  19                         \n"
               ++"  18                         \n"
               ++"  17                         \n"
               ++"  16        *       *        \n"
               ++"  15                         \n"
               ++"  14                         \n"
               ++"  13                         \n"
               ++"  12                         \n"
               ++"  11                         \n"
               ++"  10                         \n"
               ++"y  9         *     *         \n"
               ++"   8                         \n"
               ++"   7                         \n"
               ++"   6                         \n"
               ++"   5                         \n"
               ++"   4          *   *          \n"
               ++"   3                         \n"
               ++"   2                         \n"
               ++"   1           * *           \n"
               ++"   0            *            \n"
               ++"  -1                         \n"
               ++"  -2                         \n"
               ++"  -3                         \n"
               ++"  -12 -9 -6 -3  0  3  6  9 12\n"
               ++"                x\n")
        ,eqTest (run2string "y=cube x")
         "==" ("\n  21                               \n"
               ++"  20                               \n"
               ++"  19                               \n"
               ++"  18                               \n"
               ++"  17                               \n"
               ++"  16                               \n"
               ++"  15                               \n"
               ++"  14                               \n"
               ++"  13                               \n"
               ++"  12                               \n"
               ++"  11                               \n"
               ++"  10                               \n"
               ++"   9                               \n"
               ++"   8           *                   \n"
               ++"   7                               \n"
               ++"y  6                               \n"
               ++"   5                               \n"
               ++"   4                               \n"
               ++"   3                               \n"
               ++"   2                               \n"
               ++"   1          *                    \n"
               ++"   0         *                     \n"
               ++"  -1        *                      \n"
               ++"  -2                               \n"
               ++"  -3                               \n"
               ++"  -4                               \n"
               ++"  -5                               \n"
               ++"  -6                               \n"
               ++"  -7                               \n"
               ++"  -8       *                       \n"
               ++"  -9                               \n"
               ++"   -9-7-5-3-1 1 3 5 7 9111315171921\n"
               ++"                   x\n")
        ,eqTest (run2string "y=sine x")
         "==" ("\n   10                     \n"
               ++"    9     *      *     *  \n"
               ++"    8           *         \n"
               ++"    7      *              \n"
               ++"    6                 *   \n"
               ++"    5*                    \n"
               ++"    4                   * \n"
               ++"    3                     \n"
               ++"    2    *                \n"
               ++"    1             *       \n"
               ++"y   0          *          \n"
               ++"   -1       *             \n"
               ++"   -2                *    \n"
               ++"   -3                     \n"
               ++"   -4 *                   \n"
               ++"   -5                    *\n"
               ++"   -6   *                 \n"
               ++"   -7              *      \n"
               ++"   -8         *           \n"
               ++"   -9  *     *      *     \n"
               ++"  -10                     \n"
               ++"   -10 -7 -4 -1  2  5  8\n"
               ++"               x\n")          ]

-- helpers for testing below, NOT something you have to implement
square x = x^2
cube x = x^3
sine :: Int -> Int
sine x = truncate (10 * (sin (fromIntegral x)))
-- end of helpers for testing
